
#include <vcl.h>
#include <stdio.h>
#include <jpeg.hpp>


/*************************************/
/* External function declarations    */
/*************************************/

void CalculateHeigth(int panHeigth,
                     int panWidth,
                     int hdTVHeigth,
                     int hdTVWidth,
                     int cHeigth,
                     int cWidth,
                     bool constrPropChecked,
                     float aspectRatio,
                     char *output)
{
   if (aspectRatio<0.47f)
   {
     /* Probably panorama format normal */
     sprintf(output, "%d", panHeigth);
     return;
   }
   if (aspectRatio<0.61f)
   {
     /* Probably HDTV format normal */
     sprintf(output, "%d", hdTVHeigth);
     return;
   }
   if (aspectRatio<1.0f)
   {
     /* Probably Normal format normal */
     sprintf(output, "%d", cHeigth);
     return;
   }
   if (aspectRatio<1.65f)
   {
     /* Probably Normal format tilted */
     if (constrPropChecked == true)
     {
       sprintf(output, "%d", (int)(cHeigth*aspectRatio+0.5f));
     }
     else
     {
       sprintf(output, "%d", cWidth);
     }
     return;
   }
   if (aspectRatio<2.15f)
   {
     /* Probably HDTV format tilted */
     if (constrPropChecked == true)
     {
       sprintf(output, "%d", (int)(hdTVHeigth*aspectRatio+0.5f));
     }
     else
     {
       sprintf(output, "%d", hdTVWidth);
     }
     return;
   }
   if (aspectRatio>=2.15f)
   {
     /* Probably Panorama format tilted */
     if (constrPropChecked == true)
     {
       sprintf(output, "%d", (int)(panHeigth*aspectRatio+0.5f));
     }
     else
     {
       sprintf(output, "%d", panWidth);
     }
     return;
   }
}

void CalculateWidth(int panHeigth,
                    int panWidth,
                    int hdTVHeigth,
                    int hdTVWidth,
                    int cHeigth,
                    int cWidth,
                    bool constrPropChecked,
                    float aspectRatio,
                    char *output)
{
   if (aspectRatio<0.47f)
   {
     /* Probably panorama format normal */
     if (constrPropChecked == true)
     {
       sprintf(output, "%d", (int)(panHeigth*(1.0f/aspectRatio)+0.5f));
     }
     else
     {
       sprintf(output, "%d", panWidth);
     }
     return;
   }
   if (aspectRatio<0.61f)
   {
     /* Probably HDTV format normal */
     if (constrPropChecked == true)
     {
       sprintf(output, "%d", (int)(hdTVHeigth*(1.0f/aspectRatio)+0.5f));
     }
     else
     {
       sprintf(output, "%d", hdTVWidth);
     }
     return;
   }
   if (aspectRatio<1.0f)
   {
     /* Probably Normal format normal */
     if (constrPropChecked == true)
     {
       sprintf(output, "%d", (int)(cHeigth*(1.0f/aspectRatio)+0.5f));
     }
     else
     {
       sprintf(output, "%d", cWidth);
     }
     return;
   }
   if (aspectRatio<1.65f)
   {
     /* Probably Normal format tilted */
     sprintf(output, "%d", cHeigth);
     return;
   }
   if (aspectRatio<2.15f)
   {
     /* Probably HDTV format tilted */
     sprintf(output, "%d", hdTVHeigth);
     return;
   }
   if (aspectRatio>=2.15f)
   {
     /* Probably Panorama format tilted */
     sprintf(output, "%d", panHeigth);
     return;
   }
}

// Array length of string must be 1024
bool Replace(char *string,
             char *stringToRepl,
             char *replString
            )
{
  AnsiString aString;
  int pos;


  aString=string;
  if ((pos=aString.Pos(stringToRepl)) != 0)
  {
     pos--;
     memmove(string+pos+strlen(replString),
             string+pos+strlen(stringToRepl),
             1024-max(strlen(stringToRepl), strlen(replString)) - pos
            );
     memcpy(string+pos, replString, strlen(replString));
     return(true);
  }
  else
  {
    return(false);
  }
}

void CopyFile(char *inFile, char *outFile)
{
  FILE *fin;
  FILE *fout;
  unsigned char buf[1024];
  size_t nrRead;

  if (strcmp(inFile, outFile) == 0)
  {
    /* File has to be copied to itself */
    return;
  }
  else
  if ((fin = fopen(inFile, "rb")) == NULL)
  {
    /* Error opening */
    MessageBox(NULL, inFile, "Error opening for reading:", MB_OK);
    return;
  }
  if ((fout = fopen(outFile, "wb")) == NULL)
  {
    /* Error opening */
    MessageBox(NULL, outFile, "Error opening for writing:", MB_OK);
    return;
  }

  do
  {
    nrRead = fread(buf, 1, 1024, fin);
    fwrite(buf, 1, 1024, fout);
  } while (nrRead == 1024);

  fclose(fin);
  fclose(fout);
}

void ResizeJPEG(char *inFile,
                char *outFile,
                int width,
                int height,
                int quality,
                bool justcopy
               )
{
  int x,y;
  Graphics::TBitmap *Bitmap1 = new Graphics::TBitmap();
  TJPEGImage *jp = new TJPEGImage();
  float ratio;

  if (justcopy)
  {
    CopyFile(inFile, outFile);
    delete Bitmap1;
    delete jp;
    return;
  }


  jp->Performance = jpBestQuality;
  jp->LoadFromFile(inFile);
  if (((jp->Height)<=height) && ((jp->Width)<=width))
  {
    // Original image already smaller than wanted image
    Bitmap1->Assign(jp);
    jp->Assign(Bitmap1);
    jp->CompressionQuality=quality;
    jp->ProgressiveEncoding=True;
    jp->Compress();
    jp->SaveToFile(outFile);
    delete jp;
    delete Bitmap1;
    return;
  }

  // JPEG to BitMap
  Bitmap1->Assign(jp);
  Bitmap1->PixelFormat=pf24bit;

  // Resize width
  if (width<jp->Width)
  {
    ratio=(float)(jp->Width)/(float)(width);
    for (y=0; y<Bitmap1->Height; y++)
    {
      Byte* ptr;
      ptr = (Byte *)(Bitmap1->ScanLine[y]);
      for (x=0; x<width; x=x+1)
      {
         *(ptr+3*x)  =*(ptr+3*(int)(x*ratio+0.5f)  );
         *(ptr+3*x+1)=*(ptr+3*(int)(x*ratio+0.5f)+1);
         *(ptr+3*x+2)=*(ptr+3*(int)(x*ratio+0.5f)+2);
      }
    }
    Bitmap1->Width=width;
  }

  // Resize height
  if (height<jp->Height)
  {
    ratio=(float)(jp->Height)/(float)(height);
    for (y=0; y<height; y++)
    {
      Byte* ptr1;
      Byte* ptr2;

      ptr1 = (Byte *)(Bitmap1->ScanLine[y]);
      ptr2 = (Byte *)(Bitmap1->ScanLine[(int)(y*ratio+0.5f)]);
      memmove(ptr1, ptr2, 3*(Bitmap1->Width));
    }
    Bitmap1->Height=height;
  }

  // Bitmap back to JPEG and Save
  jp->Assign(Bitmap1);
  jp->CompressionQuality=quality;
  jp->ProgressiveEncoding=True;
  jp->Compress();
  jp->SaveToFile(outFile);
  delete Bitmap1;
  delete jp;
}

